from .base import BaseWrapper
from .query import PDPGetLayout
from functools import cached_property
from urllib.parse import urlparse
import json
import requests

class Product(BaseWrapper):
    def __init__(self, link=None, shop_domain=None, product_key=None, **kwargs):
        super().__init__('pdpGetLayout')
        self.link = link
        self.shop_domain = shop_domain
        self.product_key = product_key
        self.kwargs = kwargs
        self.error = None
        self.endpoint = 'https://gql.tokopedia.com/graphql/PDPGetLayoutQuery'

    def __get_shop_domain(self):
        if self.link and not self.shop_domain:
            parsed_link = urlparse(self.link)
            path = parsed_link.path.strip('/')
            self.shop_domain = path.split('/')[0]

        return self.shop_domain

    def __get_product_key(self):
        if self.link and not self.product_key:
            parsed_link = urlparse(self.link)
            path = parsed_link.path.strip('/')
            self.product_key = path.split('/')[1]

        return self.product_key

    def __get_product_info(self):
        payload = [
            {
                'operationName': 'PDPGetLayoutQuery',
                'query': PDPGetLayout,
                'variables': {
                    'apiVersion': 1,
                    'productKey': self.__get_product_key(),
                    'shopDomain': self.__get_shop_domain()
                }
            }
        ]
        response = requests.post(url=self.endpoint, json=payload, headers=self.headers)
        
        data = json.loads(response.text)
        if 'errors' in data[0]:
            self.error = data[0]['errors']

        self.data = data[0]['data']['pdpGetLayout']
        return self.data

    def __search_product_component(self, key, val):
        return next((filter(lambda x: x.get(key) == val, self.data['components'])), {})

    @cached_property
    def product_media_component(self):
        return next((filter(lambda x: x.get('name') == 'product_media', self.data['components'])), {})

    @cached_property
    def product_content_component(self):
        return next((filter(lambda x: x.get('name') == 'product_content', self.data['components'])), {})

    @cached_property
    def product_detail_component(self):
        return next((filter(lambda x: x.get('name') == 'product_detail', self.data['components'])), {})    

    def __search_product_detail_content(self, title):
        return next((filter(lambda x: x.get('title') == title, self.product_detail_component['data'][0]['content'])), {})

    @cached_property
    def serialize(self):
        self.__get_product_info()
        if not self.data or self.error:
            return None

        return {
            'meta': {
                'pdp_session': self.data['pdpSession'],
                'request_id': self.data['requestID'],
                'product_url': self.data['basicInfo']['url']
            },
            'product': {
                'id': self.data['basicInfo']['id'],
                'name': self.product_content_component['data'][0]['name'],
                'alias': self.data['basicInfo']['alias'],
                'description': self.__search_product_detail_content('Deskripsi')['subtitle'] if self.__search_product_detail_content('Deskripsi') else None,
                'condition': self.__search_product_detail_content('Kondisi')['subtitle'] if self.__search_product_detail_content('Kondisi') else None,
                'weight': self.__search_product_detail_content('Berat Satuan')['subtitle'] if self.__search_product_detail_content('Berat Satuan') else None,
                'category': self.__search_product_detail_content('Kategori')['subtitle'] if self.__search_product_detail_content('Kategori') else None,
                'images': [image['urlOriginal'] for image in self.product_media_component['data'][0]['media'] if image['type'] == 'image'],
                'price': self.product_content_component['data'][0]['price']['value'],
                'stock': self.product_content_component['data'][0]['stock']['value'],
            },
            'shop': {
                'id': self.data['basicInfo']['shopID'],
                'name': self.data['basicInfo']['shopName'],
            }
        }
        return self.data


PDPGetLayout = """
fragment ProductVariant on pdpDataProductVariant {
  errorCode
  parentID
  defaultChild
  sizeChart
  totalStockFmt
  variants {
    productVariantID
    variantID
    name
    identifier
    option {
      picture {
        urlOriginal: url
        urlThumbnail: url100
        __typename
      }
      productVariantOptionID
      variantUnitValueID
      value
      hex
      stock
      __typename
    }
    __typename
  }
  children {
    productID
    price
    priceFmt
    optionID
    optionName
    productName
    productURL
    picture {
      urlOriginal: url
      urlThumbnail: url100
      __typename
    }
    stock {
      stock
      isBuyable
      stockWordingHTML
      minimumOrder
      maximumOrder
      __typename
    }
    isCOD
    isWishlist
    campaignInfo {
      campaignID
      campaignType
      campaignTypeName
      campaignIdentifier
      background
      discountPercentage
      originalPrice
      discountPrice
      stock
      stockSoldPercentage
      startDate
      endDate
      endDateUnix
      appLinks
      isAppsOnly
      isActive
      hideGimmick
      isCheckImei
      minOrder
      __typename
    }
    thematicCampaign {
      additionalInfo
      background
      campaignName
      icon
      __typename
    }
    __typename
  }
  __typename
}

fragment ProductMedia on pdpDataProductMedia {
  media {
    type
    urlOriginal: URLOriginal
    urlThumbnail: URLThumbnail
    urlMaxRes: URLMaxRes
    videoUrl: videoURLAndroid
    prefix
    suffix
    description
    variantOptionID
    __typename
  }
  videos {
    source
    url
    __typename
  }
  __typename
}

fragment ProductHighlight on pdpDataProductContent {
  name
  price {
    value
    currency
    __typename
  }
  campaign {
    campaignID
    campaignType
    campaignTypeName
    campaignIdentifier
    background
    percentageAmount
    originalPrice
    discountedPrice
    originalStock
    stock
    stockSoldPercentage
    threshold
    startDate
    endDate
    endDateUnix
    appLinks
    isAppsOnly
    isActive
    hideGimmick
    __typename
  }
  thematicCampaign {
    additionalInfo
    background
    campaignName
    icon
    __typename
  }
  stock {
    useStock
    value
    stockWording
    __typename
  }
  variant {
    isVariant
    parentID
    __typename
  }
  wholesale {
    minQty
    price {
      value
      currency
      __typename
    }
    __typename
  }
  isCashback {
    percentage
    __typename
  }
  isTradeIn
  isOS
  isPowerMerchant
  isWishlist
  isCOD
  isFreeOngkir {
    isActive
    __typename
  }
  preorder {
    duration
    timeUnit
    isActive
    preorderInDays
    __typename
  }
  __typename
}

fragment ProductCustomInfo on pdpDataCustomInfo {
  icon
  title
  isApplink
  applink
  separator
  description
  __typename
}

fragment ProductInfo on pdpDataProductInfo {
  row
  content {
    title
    subtitle
    applink
    __typename
  }
  __typename
}

fragment ProductDetail on pdpDataProductDetail {
  content {
    title
    subtitle
    applink
    showAtFront
    isAnnotation
    __typename
  }
  __typename
}

fragment ProductDataInfo on pdpDataInfo {
  icon
  title
  isApplink
  applink
  content {
    icon
    text
    __typename
  }
  __typename
}

fragment ProductSocial on pdpDataSocialProof {
  row
  content {
    icon
    title
    subtitle
    applink
    type
    rating
    __typename
  }
  __typename
}

query PDPGetLayoutQuery($shopDomain: String, $productKey: String, $layoutID: String, $apiVersion: Float, $userLocation: pdpUserLocation, $extParam: String) {
  pdpGetLayout(shopDomain: $shopDomain, productKey: $productKey, layoutID: $layoutID, apiVersion: $apiVersion, userLocation: $userLocation, extParam: $extParam) {
    requestID
    name
    pdpSession
    basicInfo {
      alias
      createdAt
      isQA
      id: productID
      shopID
      shopName
      minOrder
      maxOrder
      weight
      weightUnit
      condition
      status
      url
      needPrescription
      catalogID
      isLeasing
      isBlacklisted
      menu {
        id
        name
        url
        __typename
      }
      category {
        id
        name
        title
        breadcrumbURL
        isAdult
        isKyc
        minAge
        detail {
          id
          name
          breadcrumbURL
          isAdult
          __typename
        }
        __typename
      }
      txStats {
        transactionSuccess
        transactionReject
        countSold
        paymentVerified
        itemSoldFmt
        __typename
      }
      stats {
        countView
        countReview
        countTalk
        rating
        __typename
      }
      __typename
    }
    components {
      name
      type
      position
      data {
        ...ProductMedia
        ...ProductHighlight
        ...ProductInfo
        ...ProductDetail
        ...ProductSocial
        ...ProductDataInfo
        ...ProductCustomInfo
        ...ProductVariant
        __typename
      }
      __typename
    }
    __typename
  }
}
"""



"""
Tokopedia API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the Tokopedia GraphQL API.
:copyright: (c) 2022-present munawariz
:license: MIT, see LICENSE for more details.
"""

__title__ = 'tokopedia-wrapper'
__author__ = 'munawariz'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023-present munawariz'
__version__ = '0.2'


class BaseWrapper:
    def __init__(self, akamai=None):
        self.generate_headers(akamai)

    def generate_headers(self, akamai=None):
        self.headers = {
            'Content-Type': 'application/json'
        }

        if akamai:
            self.headers['X-TKPD-AKAMAI'] = akamai
        
        return self.headers

    def serialize(self):
        raise NotImplementedError